function [PSD, frequenciesy, frequenciesx, totalsegments] = estimateSpectrumWOSA(I, resolution, spectralparams)

% Copyright 2015 Kimmo Kartasalo
% Tampere University of Technology, Tampere, Finland
% http://www.tut.fi/cytospectre
% Email: kimmo.kartasalo@tut.fi
% 
% This file is part of CytoSpectre.
% 
% CytoSpectre is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% CytoSpectre is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with CytoSpectre.  If not, see <http://www.gnu.org/licenses/>.

I = double(I);
inputsize = size(I);

% Create circular 2D Hann window via linear interpolation from the 1D window.
windowlength1D = ceil(spectralparams.windowinterpolation*max(spectralparams.windowsize)/2);
window1D = hanning(2*windowlength1D + 1);
windowradial = [window1D(windowlength1D+1:end); 0];
radialpoints = (0:windowlength1D+1)'/(windowlength1D+1);
queryxi = repmat(((0:spectralparams.windowsize(1)-1)-(spectralparams.windowsize(1)-1)/2)/((spectralparams.windowsize(1)-1)/2), [spectralparams.windowsize(2) 1]);
queryyi = repmat(((0:spectralparams.windowsize(2)-1)-(spectralparams.windowsize(2)-1)/2)'/((spectralparams.windowsize(2)-1)/2), [1 spectralparams.windowsize(1)]); 
querypoints = sqrt(queryxi.^2+queryyi.^2);
querypoints(querypoints>1) = 1;
window = querypoints;
window(:) = interp1(radialpoints, windowradial, querypoints(:), 'linear');

% Calculate number of segments, adjust overlap and calculate step size.
numsegments = round((inputsize - spectralparams.overlap.*spectralparams.windowsize)./(spectralparams.windowsize-spectralparams.overlap.*spectralparams.windowsize) + 0.2);
spectralparams.overlap = (spectralparams.windowsize.*numsegments - inputsize)./(spectralparams.windowsize.*numsegments - spectralparams.windowsize + eps);
step = (1-spectralparams.overlap).*spectralparams.windowsize;

% Calculate mean and variance of the input image. If there is only one
% segment, the effect of the window has to be taken into account since it
% is not cancelled during segment averaging.
if all(numsegments == 1)
    directestimate = (window > 0.05*max(window(:))).*(I(1:spectralparams.windowsize(1),1:spectralparams.windowsize(2)));
    nonzerodirectestimate = directestimate(directestimate ~= 0);
    mu = mean(nonzerodirectestimate(:));
    vari = var(nonzerodirectestimate(:));
else
    mu = mean(I(:));
    vari = var(I(:));
end

% Estimate spectrum for each segment, compute PSD as the squared modulus of
% the spectral values and sum the segments.
totalsegments = numsegments(1)*numsegments(2);
segments = zeros(spectralparams.fftlength(1),spectralparams.fftlength(2),1);
for segmentx = 1:numsegments(2)
    indx = floor((segmentx - 1)*step(2));
    for segmenty = 1:numsegments(1)
        indy = floor((segmenty - 1)*step(1));
        segments_this = fft2(window.*(I(indy+1:indy+spectralparams.windowsize(1),indx+1:indx+spectralparams.windowsize(2)) - mu), spectralparams.fftlength(1), spectralparams.fftlength(2));
        segments = segments + real(segments_this).^2 + imag(segments_this).^2;
    end
end

% Form frequency axes.
frequenciesy = resolution(1)/spectralparams.fftlength(1)*(ceil(-spectralparams.fftlength(1)/2):ceil(spectralparams.fftlength(1)/2) - 1);
frequenciesx = resolution(2)/spectralparams.fftlength(2)*(ceil(-spectralparams.fftlength(2)/2):ceil(spectralparams.fftlength(2)/2) - 1);

% Form PSD estimate as the mean of the individual segments (just divide by
% the number of segments, as summing was already done within the for loop).
PSD = segments/totalsegments;

% Shift origin to middle and remove leftmost and topmost lines of the spectrum if
% necessary.
PSD = fftshift(PSD);
if mod(size(PSD,1),2) == 0
    PSD = PSD(2:end,:);
    frequenciesy = frequenciesy(2:end);
end
if mod(size(PSD,2),2) == 0
    PSD = PSD(:,2:end);
    frequenciesx = frequenciesx(2:end);
end

% Force the sum of the spectrum to correspond to variance of the input (Parseval).
varcorrection = vari/sum(PSD(:));
PSD = varcorrection*PSD;
