function [maindirection, transversedirection] = findDirections(PSD_polar, frequencies_polar, lowestfreq, highestfreq, segmentpercentage, alpha)

% Copyright 2015 Kimmo Kartasalo
% Tampere University of Technology, Tampere, Finland
% http://www.tut.fi/cytospectre
% Email: kimmo.kartasalo@tut.fi
% 
% This file is part of CytoSpectre.
% 
% CytoSpectre is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% CytoSpectre is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with CytoSpectre.  If not, see <http://www.gnu.org/licenses/>.

% Calculate mean orientation as a function of frequency.
angles = linspace(0,2*pi,size(PSD_polar,1))';
meanorientation = zeros(1,size(PSD_polar,2));
for freqind=1:size(PSD_polar,2)
    lowmarginal_polar = PSD_polar(:,freqind);
    loworientationdistribution = lowmarginal_polar/sum(lowmarginal_polar);
    meanorientation(freqind) = 0.5*circ_mean(angles, loworientationdistribution);
end
meanorientation = circ_rad2ang(meanorientation);
meanorientation(meanorientation < 0) = meanorientation(meanorientation < 0) + 180;

% Remove forbidden frequencies.
meanorientation_valid = meanorientation(frequencies_polar >= lowestfreq & frequencies_polar <= highestfreq);
frequencies_polar_valid = frequencies_polar(frequencies_polar >= lowestfreq & frequencies_polar <= highestfreq);

% Transform the mean orientations to differences relative to the first value
% to avoid problems near 0 and 180 degrees.
meanorientation_diffs = abs(meanorientation_valid-meanorientation_valid(1));
% Differences over 90 degrees are actually 180 - difference.
meanorientation_diffs(meanorientation_diffs > 90) = 180 - meanorientation_diffs(meanorientation_diffs > 90); 

% Set length of data segment. Force it to be odd and at most half of the whole valid data length.
segmentlength = round(segmentpercentage*length(frequencies_polar));
if segmentlength > round(length(frequencies_polar_valid)/2)
    segmentlength = round(length(frequencies_polar_valid)/2);
end
if mod(segmentlength,2) == 0
    segmentlength = segmentlength + 1;
end

% Initialize lowpass-highpass transition frequency at the highest frequency.
transitionfrequency = highestfreq;
% Loop through the data to detect an abrupt change in orientation, the
% frequency where this happens is the lowpass-highpass transition
% frequency.
startpos = ceil(segmentlength/2) + 1;
endpos = length(frequencies_polar_valid) - floor(segmentlength/2);
previousdata = meanorientation_diffs(1:startpos-1);
for ind = startpos:1:endpos
    thissegment = meanorientation_diffs(ind-floor(segmentlength/2):ind+floor(segmentlength/2));
    p = ranksum(thissegment,previousdata);
    if p < alpha
        transitionfrequency = frequencies_polar_valid(ind-1);
        break;
    else
        previousdata = [previousdata meanorientation_diffs(ind)];
    end
end

% Extract the lowpass region of the spectrum.
firstlowind = round(length(frequencies_polar)*(lowestfreq/max(frequencies_polar)));
lastlowind = round(length(frequencies_polar)*(transitionfrequency/max(frequencies_polar)));
if firstlowind < 1
    firstlowind = 1;
end
if lastlowind > length(frequencies_polar)
    lastlowind = length(frequencies_polar);
end
lowmarginal_polar = sum(PSD_polar(:,firstlowind:lastlowind),2);
loworientationdistribution = lowmarginal_polar/sum(lowmarginal_polar);

% Calculate main directions.
maindirection = circ_rad2ang(0.5*circ_mean(angles, loworientationdistribution));
transversedirection = maindirection - 90;
if maindirection < 0
    maindirection = maindirection + 180;
end
if transversedirection < 0
    transversedirection = transversedirection + 180;
end
