function [PSD_polar_ROI,iter] = separateSpectralROI_iterative(PSD_polar, PSD_polar_prob, frequencies_polar, lowfreqcutoff, highfreqcutoff, mu_angle_init, mu_freq_init, sigma_angle_init, sigma_freq_init, convergence_threshold, max_iterations)

% Copyright 2015 Kimmo Kartasalo
% Tampere University of Technology, Tampere, Finland
% http://www.tut.fi/cytospectre
% Email: kimmo.kartasalo@tut.fi
% 
% This file is part of CytoSpectre.
% 
% CytoSpectre is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% CytoSpectre is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with CytoSpectre.  If not, see <http://www.gnu.org/licenses/>.

% Calculate prior.
prior = formGaussianPrior(size(PSD_polar), frequencies_polar, mu_angle_init, mu_freq_init, sigma_angle_init, sigma_freq_init);

% Force the probabilities to zero at low frequencies.
% Avoid overindexing.
if round(length(frequencies_polar)*(lowfreqcutoff/max(frequencies_polar))) > length(frequencies_polar)
    prior(:,1:end) = 0;
% Avoid underindexing.
elseif round(length(frequencies_polar)*(lowfreqcutoff/max(frequencies_polar))) > 0
    prior(:,1:round(length(frequencies_polar)*(lowfreqcutoff/max(frequencies_polar)))) = 0;
end

% Force the probabilities to zero at high frequencies.
% Avoid underindexing.
if round(length(frequencies_polar)*(highfreqcutoff/max(frequencies_polar))) < 1
    prior(:,1:end) = 0;
% Avoid overindexing.
elseif round(length(frequencies_polar)*(highfreqcutoff/max(frequencies_polar))) < length(frequencies_polar)
    prior(:,round(length(frequencies_polar)*(highfreqcutoff/max(frequencies_polar))):end) = 0;
end

% Calculate posterior PSD.
posteriorPSD = prior.*PSD_polar_prob;
% Threshold the PSD to obtain ROI.
PSD_polar_ROI = PSD_polar.*(posteriorPSD > 0.5);

% Iterate to adjust parameters.
mu_angle_prev = mu_angle_init;
sigma_angle_prev = sigma_angle_init;
iter = 1;
while true
    % Stop if entire ROI is zero.
    if all(PSD_polar_ROI == 0)
        PSD_polar_ROI = [];
        break;
    end
    
    % Calculate change in mean angle and angular std.
    orientation = analyzeOrientations_cirstatTB(PSD_polar_ROI);
    mu_angle = orientation.mean;
    sigma_angle = circ_rad2ang(orientation.astd);
    changeinangle = abs(mu_angle_prev - mu_angle);
    if changeinangle > 90
        changeinangle = 180 - changeinangle;
    end
    changeinsigma = abs(sigma_angle_prev - sigma_angle);
    % Check if both changes are below convergence threshold or if the
    % maximum number of iterations was exceeded.
    if (changeinangle < convergence_threshold) && (changeinsigma < convergence_threshold)
        break;
    elseif iter >= max_iterations
        break;
    else
        mu_angle_prev = mu_angle;
        sigma_angle_prev = sigma_angle;
        iter = iter + 1;
    end
    
    % Form new prior, posterior PSD and thresholded ROI.
    prior = formGaussianPrior(size(PSD_polar), frequencies_polar, mu_angle, mu_freq_init, sigma_angle, sigma_freq_init);
    if round(length(frequencies_polar)*(lowfreqcutoff/max(frequencies_polar))) > length(frequencies_polar)
        prior(:,1:end) = 0;
    elseif round(length(frequencies_polar)*(lowfreqcutoff/max(frequencies_polar))) > 0
        prior(:,1:round(length(frequencies_polar)*(lowfreqcutoff/max(frequencies_polar)))) = 0;
    end
    if round(length(frequencies_polar)*(highfreqcutoff/max(frequencies_polar))) < 1
        prior(:,1:end) = 0;
    elseif round(length(frequencies_polar)*(highfreqcutoff/max(frequencies_polar))) < length(frequencies_polar)
        prior(:,round(length(frequencies_polar)*(highfreqcutoff/max(frequencies_polar))):end) = 0;
    end
    posteriorPSD = prior.*PSD_polar_prob;
    PSD_polar_ROI = PSD_polar.*(posteriorPSD > 0.5);
end
